// includes defines, globals

#include "C:\Program Files\PICCa\Projects\lf2523\main.h"

// globals

////////////// time stuff
int1 secflag; // set by ISR on the second, cleared by main when read
int8 seconds;
int8 minutes;
int16 hours;
int8 even; // temp for toggles
#define NUM_ONESHOTS 8
int8 oneshots[NUM_ONESHOTS];

////////////////////////// assorted other globals
int8 i; // used all over main, loop index

#DEFINE AD_CH 4

int16 pwmduty = 128;
int16 adres[AD_CH]; // size = number of a/d channels
int16 adresb[AD_CH];
///////////////////////// isr stuff -- warning, different thread(s)
// our version of "hungrarian notation" is to preface with i on isr variables.
int8 ii; // used in ISR threads, loop index
int8 iadidx;



//********************************//////// subroutines
void bufferAD()
{
// this could get fancier if one cares whether all the samples are from
// the same channel sweep or not -- or you could plan to use adres sometime
// when you knew the sweep was over anyway.
 DISABLE_INTERRUPTS(INT_AD);
 for (i=0;i<AD_CH;i++) adresb[i]= adres[i];
 ENABLE_INTERRUPTS(INT_AD);
}
//////////////////////////

//////////////////////////
void ZeroTime()
{
 secflag = FALSE;
 seconds = minutes = 0;
 hours = 0;
 for (i=0;i<NUM_ONESHOTS;i++) oneshots[i] = 0; // clear out timers
 for (i=0;i<AD_CH;i++) adres[i] = 0; // clear out a/d results

 even = 0; // toggle flag
 iadidx = 0; // current a/d channel for aq
}
//////////////////////////
/* 

Set hardware and memory up here.  This will have to change when you use
a different PIC, of course, to match the hardware you've got and what's
hooked to what.

The hardware guy should fix things so the way the pic powers up doesn't do
bad things, and that's easy -- pretty much everything comes up float/input,
so you can have pullups-pulldowns as required for this.

The software guy can set thing up as desired before changing any i/o pins
to output.  No smoke need be released from anything, ever.

Not set up now to use priority interrupts (which can interrupt low pri ones)
IF that is changed, make sure no interruptable interrupt shares variables with
a high priority one.

Am not using "fast" either for similar reasons.  Use this till something better
is really needed, it's pretty quick.

*/
void init()
{
   setup_oscillator(OSC_32MHZ);  // try internal RC with pll
// Get rid of the xtal -- will it be accurate?
// Well at any rate, it does work, and speed seems good
   ZeroTime();  // runtime starts now -- all time stuff cleared, mem setup

// we use fast_io and do these things ourself
   set_tris_a(0b00001111);  // 4 a/d inputs, clk? now internal RC
   port_b_pullups(TRUE);  // keeps down noise on inputs
   set_tris_b(0b11111111);
   set_tris_c(0b10010010); // rs 232, oscillator, pwm/ccp1
   
   setup_adc_ports(AN0_TO_AN3 | VSS_VDD);  // 4 channels
   setup_adc(ADC_CLOCK_DIV_32 | ADC_TAD_MUL_4); // as fast as possible?
   set_adc_channel(iadidx); // start on chnl zero
   enable_interrupts(INT_AD); // a/d interrupts do the channel sweep
//   setup_spi(SPI_SS_DISABLED); // temp
   setup_wdt(WDT_OFF);        // temp  
   setup_timer_0(RTCC_OFF);   // temp

   setup_timer_1(T1_EXTERNAL_RTC); // oscillate with 32768 hz clock xtal
   enable_interrupts(INT_TIMER1); // enables the timer1 interrupt

   setup_timer_3(T3_INTERNAL | T3_DIV_BY_8); // counts at 1 mhz
   enable_interrupts(INT_TIMER3); // allow the interrupt
   
   setup_timer_2(T2_DIV_BY_1,0x7f,1); // ~62.25 khz for pwm, 9 bit rez
   setup_ccp1(CCP_PWM); // start pwm output
   set_pwm1_duty (pwmduty); // start with sq wave (diagnostic)
 
   setup_comparator(NC_NC_NC_NC); // temp
   setup_vref(FALSE); // temp
 
   enable_interrupts(GLOBAL); // start them now, see below
}

//////////////////////////////////********** interrupts *******************
#int_timer1 // flag each passing second
void timer_1_interrupt()
{ // important to reload timer in a timely way
#byte TMR1H = 0xfcf  // tested if these work in access bank, NOT!
 bit_set(TMR1H,7);   // makes a bsf, but no TMR1L access, and no effect
 secflag = TRUE;     // set flag for background, let it do the calx for HMS,
 //can then read them without ever having a carry error during capture
}
//////////////////////////////////**********
#int_timer3 // used for faster timings, 10 ms oneshots
void timer_3_interrupt()
{ // should get here at 100 hz more or less, reload timer, do oneshots
set_timer3(55535); // set to count 10k times per interrupt
read_adc(ADC_START_ONLY); // setup elesewhere, start a sweep here

for (ii=0;ii<NUM_ONESHOTS;ii++) 
   if (oneshots[ii]) oneshots[ii]--; // count down if not zero

even++;
if (even & 1) // setup a toggle, diagnostic code
   {
    OUTPUT_HIGH(PIN_C3);// green led
    OUTPUT_LOW(PIN_C5);
   } else
   {
    OUTPUT_HIGH(PIN_C5); // red led
    OUTPUT_LOW(PIN_C3);
   }

}// end timer3 int
///////////////////////////////
#int_AD
void AD_interrupt() // gather a/d channels into an array for bckgnd
{ // sweep triggered by timer at 100hz for now
 adres[iadidx++] = read_adc(ADC_READ_ONLY) >> 4; // get a/d channel into array
 // shift to right-justify, 0-4095 range results
 iadidx &= (AD_CH-1); // limit it to the channels we've set up  @@@ #defines?
 set_adc_channel(iadidx); // point to next channel to acquire
 if (0!=iadidx) read_adc(ADC_START_ONLY); // get next chnl in sweep if not done
}
///////////////////////////////




/***************************************************************************/
// Main.  This loops forever calling application state machines in turn
/////////////////////////////////////////////////////////////////////////////
void main()
{
 init();  // do hardware & memory init

  for(;;) // forever loop, call app state machines and track runtime
  {
//  pwmduty = adres[0] >> 3; // 12 bits to 9 bits  
   OUTPUT_TOGGLE (PIN_A6);  // Show we were here, pin 10 on 18lf2523
  
  if (secflag)
  {
   secflag = FALSE; // we saw it
   if (++seconds >= 60)
   { // minute boundary
    seconds = 0;  // start new minute
    if (++minutes >= 60)
    {
     minutes = 0;
     hours++;
    } // hour boundary end
   } // minute boundary end


// debug here once/second, app calls maybe
// else call apps after seconds loop -- fastest

   pwmduty = adres[0] >> 3; // 12 bits to 9 bits
   set_pwm1_duty (pwmduty); // start with a/d input for testing

   printf("\r\nRuntime:%Lu:%02u:%02u",hours,minutes, seconds);


//   printf("\tAD0:%Lu  AD1:%Lu",adres[0],adres[1]);
//   printf("  Even:%03u",even);
   
  } // end if secflag
  
////////////////// fast app calls here  //////////////////////////////




  } // forever loop end
// can't get here unless disaster
} // end of main

